using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;

public class Helper
{
	public T NumToEnum<T>(int number)
	{
	   return (T)System.Enum.ToObject(typeof(T), number);
	}
}

public static class Helper_Static
{
	#region 2D Toolkit
	public static bool ContainAnim(this tk2dAnimatedSprite value, string animName)
	{
		if (string.IsNullOrEmpty(animName)) return false;
		
		tk2dSpriteAnimationClip [] clips = value.anim.clips;
		for (int i=0; i<clips.Length; i++)
		{
			tk2dSpriteAnimationClip clip = clips[i];
			string clipName = clip.name;
			
			if ( animName.Length != clipName.Length )
				continue;
			
			if (clipName.Equals ( animName ) )
			{
				return true;
			}
		}
		
		return false;
	}
	
	/// <summary>
	/// Returns the length (seconds) of the current animation clip.
	/// </summary>
	public static float CurrentClipLength(this tk2dAnimatedSprite value)
	{
		tk2dSpriteAnimationClip currentClip = value.CurrentClip;
		return ((float)currentClip.frames.Length / (float)currentClip.fps);
	}
	
	/// <summary>
	/// Find an animation clip given the name, and returns its length (seconds).
	/// </summary>
	public static float ClipLength(this tk2dAnimatedSprite value, string animName)
	{
		tk2dSpriteAnimation valueAnim = value.anim;
		tk2dSpriteAnimationClip _clip = valueAnim.clips[valueAnim.GetClipIdByName(animName)];
		return ((float)_clip.frames.Length / (float)_clip.fps);
	}
	
	/// <summary>
	/// Returns the length (seconds) of an animation clip.
	/// </summary>
	public static float Length(this tk2dSpriteAnimationClip value)
	{
		return ((float)value.frames.Length / (float)value.fps);
	}
	
	/// <summary>
	/// Returns an animation clip given the name.
	/// </summary>
	public static tk2dSpriteAnimationClip GetClipByName(this tk2dAnimatedSprite value, string animName)
	{
		 return value.anim.clips[value.anim.GetClipIdByName(animName)];
	}
	
	/// <summary>
	/// Play an animation one at a time.
	/// </summary>
	/// 
	public static void PlayOnce(this tk2dAnimatedSprite value, string animName)
	{
		//if ( !value.ContainAnim ( animName ) ) return;
		
		if (!value.Playing )
		{
			value.Play(animName);
		}
		else 
		{
			string name = value.CurrentClip.name;
			if ( name.Length != animName.Length )
			{
				value.Play ( animName );
			}
			else 
			{
				if ( !name.Equals ( animName ) ) 
				{
					value.Play(animName);
				}
			}
		}
	}
	
	public static void PlayOnce ( this tk2dAnimatedSprite value, string _animName, float speedMultiplier )
	{
		tk2dSpriteAnimationClip clip = value.GetClipByName ( _animName );
		float clipFps = clip.fps;
		
		// This is to prevent divide by zero error
		if ( speedMultiplier == 0 )
		{
			speedMultiplier = 0.01f;
			
#if UNITY_EDITOR
			Debug.Log ( "speed multiplier is zero, check unit attack speed?" );
#endif
		}
		
		value.Play ( clip, 0f, clipFps * 1f / speedMultiplier );
	}
	
	public static void Play(this tk2dAnimatedSprite value, string _animName)
	{
		Transform _thisTransform = value.transform;
		if (_thisTransform.position.y != 0)
		{
			_thisTransform.position = _thisTransform.position.yZeroed();
		}
		
		value.Play(_animName);
	}
	
	public static Vector2 SpriteSize(this tk2dSprite value)
	{
		// original version by aCallum
		Texture2D _tex = (Texture2D)value.GetCurrentSpriteDef().material.mainTexture;

        Vector2[] UVs = value.GetCurrentSpriteDef().uvs;

        // Get the raw uv-pixel co-ordinates of the image.
        float fX = (_tex.width) * UVs[0].x;
        float fY = (_tex.height) * UVs[2].y;
        float fX2 = (_tex.width) * UVs[1].x;;
        float fY2 = (_tex.height) * UVs[0].y;

        // Calculate the width and height of the sprite.
        float fWidth = fX2 - fX;
        float fHeight = -(fY2 - fY);
		
		return new Vector2(fWidth, fHeight);
	}
	
	public static Vector2 SpriteSize(this tk2dAnimatedSprite value)
	{
		// original version by aCallum
		Texture2D _tex = (Texture2D)value.GetCurrentSpriteDef().material.mainTexture;

        Vector2[] UVs = value.GetCurrentSpriteDef().uvs;

        // Get the raw uv-pixel co-ordinates of the image.
        float fX = (_tex.width) * UVs[0].x;
        float fY = (_tex.height) * UVs[2].y;
        float fX2 = (_tex.width) * UVs[1].x;;
        float fY2 = (_tex.height) * UVs[0].y;

        // Calculate the width and height of the sprite.
        float fWidth = fX2 - fX;
        float fHeight = -(fY2 - fY);
		
		return new Vector2(fWidth, fHeight);
	}
	
	public static float SpriteSizeMultiplier(Vector2 spriteSize, float cameraOrthoSize, float screenHeight)
	{
		float pixelRatio = spriteSize.y / screenHeight;
		float maxSpriteScale = cameraOrthoSize * pixelRatio;
		return (cameraOrthoSize / maxSpriteScale);
	}
	
	public static void SetInitialSortingOrder ( this tk2dSprite value, int _order )
	{
		if ( value != null )
		{
			value.transform.position = value.transform.position.yZeroed ();
			
			value.SortingOrder = _order;
			value.inheritSortingOrder = _order;
			value.followParentSortingOrder = false;
		}
	}
	
	public static void SetInitialSortingOrder ( this tk2dAnimatedSprite value, int _order )
	{
		if ( value != null )
		{
			value.transform.position = value.transform.position.yZeroed ();
			
			value.SortingOrder = _order;
			value.inheritSortingOrder = _order;
			value.followParentSortingOrder = false;
		}
	}
	
	public static void SetInheritSortingOrder(this tk2dSprite value, int _order )
	{
		if ( value != null )
		{
			value.transform.position = value.transform.position.yZeroed ();
			
			value.inheritSortingOrder = _order;
			value.followParentSortingOrder = true;
		}
	}
	
	public static void SetInheritSortingOrder(this tk2dAnimatedSprite value, int _order )
	{
		if ( value != null )
		{
			value.transform.position = value.transform.position.yZeroed ();
			
			value.inheritSortingOrder = _order;
			value.followParentSortingOrder = true;
		}
	}
	#endregion
	
	/// <summary>
	/// Returns the direction between two points, using only their X and Z axis.
	/// </summary>
	public static Vector3 DirectionXZ(Vector3 _startPos, Vector3 _endPos)
	{
		_startPos.y = 0;
		_endPos.y = 0;
		
		return (_endPos - _startPos).normalized;
	}
	
	public static Vector3 yZeroed(this Vector3 value)
	{
		value.y = 0;
		return value;
	}
	
	public static float ViewDirectionXZ_Right(Vector3 callerPos, Vector3 targetPos)
	{
		Vector3 dir = Helper_Static.DirectionXZ(callerPos, targetPos);
		float _dotX = Vector3.Dot(dir, Vector3.right);
		
		int _dir = 1;
		
		if (_dotX < 0)
		{
			_dir = -1;
		}
		
		return _dir;
	}
	
	public static int ViewDirectionXZ_Forward(Vector3 callerPos, Vector3 targetPos)
	{
		var dir = Helper_Static.DirectionXZ(callerPos, targetPos);
		var _dotY = Vector3.Dot(dir, Vector3.forward);
		
		if (_dotY > 0)
		{
			return 1;
		}
		else if (_dotY < 0)
		{
			return -1;
		}
		
		return 0;
	}
	
	public static string SplitAndStitchString(this string value, bool withSpace = true)
	{
		string[] stringSeperator = new string[1];
		stringSeperator[0] = "_";
		string[] seperatedString = value.Split(stringSeperator, System.StringSplitOptions.RemoveEmptyEntries);
		string fullString = default(string);
		if (seperatedString.Length > 0)
		{
			for (int i=0; i<seperatedString.Length; i++)
			{
				if (fullString != default(string) && withSpace)
				{
					fullString += " ";
				}
				fullString += seperatedString[i];
			}
		}
		
		return fullString;
	}
	
	public static string SplitAndStitchString(this string value, int[] ignoreIndexes, bool withSpace = true)
	{
		string[] stringSeperator = new string[1];
		stringSeperator[0] = "_";
		string[] seperatedString = value.Split(stringSeperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		List<string> tempSeperatedString = seperatedString.ToList();
		if (ignoreIndexes.Length > 0)
		{
			for (int i=0; i<ignoreIndexes.Length; i++)
			{
				tempSeperatedString.RemoveAt(ignoreIndexes[i]);
			}
		}
		
		string fullString = default(string);
		
		seperatedString = tempSeperatedString.ToArray();
		
		if (seperatedString.Length > 0)
		{
			for (int i=0; i<seperatedString.Length; i++)
			{
				if (fullString != default(string) && withSpace)
				{
					fullString += " ";
				}
				fullString += seperatedString[i];
			}
		}
		
		return fullString;
	}
	
	public static string[] SplitWithPredefinedSeperators(this string value)
	{
		string[] stringSeperator = new string[2];
		stringSeperator[0] = "_";
		string[] seperatedString = value.Split(stringSeperator, System.StringSplitOptions.RemoveEmptyEntries);
		
		return seperatedString;
	}
	
	public static void AddOnce<T>(this BetterList<T> value, T _item)
	{
		if (value != null)
		{
			if (!value.Contains(_item))
			{
				value.Add(_item);
			}
		}
	}
	
	public static void AddRange<T>(this BetterList<T> value, T[] _items)
	{
		if (_items != null && _items.Length > 0)
		{
			for (int i=0; i<_items.Length; i++)
			{
				T item = _items [ i ];
				if (!value.Contains(item))
				{
					value.Add(item);
				}
			}
		}
	}
	
	public static void AddRange<T>(this BetterList<T> value, BetterList<T> _items)
	{
		if (_items != null && _items.size > 0)
		{
			for (int i=0; i<_items.size; i++)
			{
				T item = _items [ i ];
				if (!value.Contains(item))
				{
					value.Add(item);
				}
			}
		}
	}
	
	public static void AddRange<T>(this BetterList<T> value, List<T> _items)
	{
		if (_items != null && _items.Count > 0)
		{
			for (int i=0; i<_items.Count; i++)
			{
				T item = _items [ i ];
				if (!value.Contains(item))
				{
					value.Add(_items[i]);
				}
			}
		}
	}
	
	/// <summary>
	/// Adds the range quick without checking whether it contain the element already or not.
	/// </summary>
	/// <param name='value'>
	/// Value.
	/// </param>
	/// <param name='_items'>
	/// _items.
	/// </param>
	/// <typeparam name='T'>
	/// The 1st type parameter.
	/// </typeparam>
	public static void AddRangeQuick<T> ( this BetterList<T> value, BetterList<T> _items )
	{
		if ( _items != null && _items.size > 0 )
		{
			for ( int i=0; i<_items.size; i++ )
			{
				T item = _items [ i ];
				value.Add ( item );
			}
		}
	}
	
	public static void AddRangeQuick<T> ( this BetterList<T> value, List<T> _items )
	{
		if ( _items != null && _items.Count > 0 )
		{
			for ( int i=0; i<_items.Count; i++ )
			{
				T item = _items [ i ];
				value.Add ( item );
			}
		}
	}
	
	public static T[] Add<T>(this T[] value, T _item)
	{
		List<T> newValue = value.ToList();
		newValue.Add(_item);
		
		return newValue.ToArray();
	}
	
	public static T[] AddRange<T>(this T[] value, T[] _items)
	{
		List<T> newValue = value.ToList();
		newValue.AddRange(_items.ToList());
		
		return newValue.ToArray();
	}
	
	public static bool Contains<T> ( this T[] value, T item )
	{
		if ( value == null || item == null ) return false;
		
		for ( int i = 0; i < value.Length; i++ )
		{
			if ( item.Equals ( value [ i ] ) )
			{
				return true;
			}
		}
		
		return false;
	}
	
	public static T[] Remove<T>(this T[] value, T _item)
	{
		int _index = System.Array.IndexOf ( value, _item );
		
		return value.RemoveAt ( _index );
	}
	
	/*public static T[] RemoveAt<T>(this T[] value, int _index)
	{
		List<T> _newValue = value.ToList ();
		_newValue.RemoveAt ( _index );
		
		return _newValue.ToArray ();
	}*/
	
	public static T[] RemoveAt<T> ( this T[] value, int _index )
	{
	    T[] _newValue = new T[ value.Length - 1 ];
	    if ( _index > 0 )
		{
			System.Array.Copy ( value, 0, _newValue, 0, _index );
		}
	
	    if ( _index < value.Length - 1 )
		{
			System.Array.Copy ( value, _index + 1, _newValue, _index, value.Length - _index - 1 );
		}
	
	    return _newValue;
	}
	
	public static T[] Insert<T> ( this T[] value, T _item, int _index )
	{
		T[] _newValue = new T[ value.Length + 1 ];
		
		for ( int i=0; i<_index; i++ )
		{
			_newValue[i] = value[i];
		}
		_newValue[ _index ] = _item;
		for ( int i= (_index + 1 ); i<_newValue.Length; i++ )
		{
			_newValue[i] = value[ i - 1 ];
		}
		
		//List<T> newValue = value.ToList();
		//newValue.Add(_item);
		
		return _newValue;
	}
	
	public static List<T> ToList<T>(this T[] value)
	{
		List<T> newT = new List<T>();
		
		for (int i=0; i<value.Length; i++)
		{
			newT.Add(value[i]);
		}
		
		return newT;
	}
	
	public static void AddOnce<T>(this List<T> value, T item)
	{
		if (item != null && !value.Contains(item))
		{
			value.Add(item);
		}
	}
	
	public static void RemoveOnce<T>(this List<T> value, T item)
	{
		if (value.Contains(item))
		{
			value.Remove(item);
		}
	}
	
	public static void AddRange<T> ( this List<T> value, T[] _items )
	{
		if ( _items != null && _items.Length > 0 )
		{
			foreach ( T _t in _items )
			{
				value.Add ( _t );
			}
		}
	}
	
	public static int ViewDirectionXZ(Vector3 _callerPos, Vector3 _targetPos)
	{
		// -1 = Left; 1 = Right
		Vector3 dir = DirectionXZ(_callerPos, _targetPos);
		float _dotX = Vector3.Dot(dir, Vector3.right);
		
		if (_dotX < 0)
		{
			return -1;
		}
		
		return 1;
	}
	
	public static int ViewDirectionXZUp(Vector3 _callerPos, Vector3 _targetPos)
	{
		// -1 = Up; 1 = Down
		Vector3 dir = DirectionXZ(_callerPos, _targetPos);
		float _dotY = Vector3.Dot(dir, Vector3.forward);
		
		if (_dotY < 0)
		{
			return -1;
		}
		
		return 1;
	}
	
	public static string ColorToHex(Color32 color)
	{
		string hex = color.r.ToString("X2") + color.g.ToString("X2") + color.b.ToString("X2");
		return hex;
	}
	
	public static Color HexToColor(string hex)
	{
		byte r = byte.Parse(hex.Substring(0,2), System.Globalization.NumberStyles.HexNumber);
		byte g = byte.Parse(hex.Substring(2,2), System.Globalization.NumberStyles.HexNumber);
		byte b = byte.Parse(hex.Substring(4,2), System.Globalization.NumberStyles.HexNumber);
		return new Color32(r,g,b, 255);
	}
	
	public static bool IsSave<T>(this T[] value)
	{
		return (value != null && value.Length > 0);
	}
	
	public static bool IsSave<T>(this List<T> value)
	{
		return (value != null && value.Count > 0);
	}
	
	public static void Swap<T> ( this List<T> value, T _item0, T _item1 )
	{
	    int _index0 = value.IndexOf ( _item0 );
		int _index1 = value.IndexOf ( _item1 );
	   
		value[ _index0 ] = _item1;
		value[ _index1 ] = _item0;
	}
	
	public static T[] SwapWithPrevious<T>(this T[] value, int _currentIndex)
	{
		if (_currentIndex > 0)
		{
			int _prevIndex = _currentIndex - 1;
			T _previous = value[_prevIndex];
			T _current = value[_currentIndex];
			
			value[_prevIndex] = _current;
			value[_currentIndex] = _previous;
		}
		
		return value;
	}
	
	public static T[] SwapWithNext<T>(this T[] value, int _currentIndex)
	{
		if (_currentIndex < value.Length - 1)
		{
			int _nextIndex = _currentIndex + 1;
			T _next = value[_nextIndex];
			T _current = value[_currentIndex];
			
			value[_nextIndex] = _current;
			value[_currentIndex] = _next;
		}
		
		return value;
	}
	
	public static void SwapWithPrevious<T>(this List<T> value, int _currentIndex)
	{
		if (_currentIndex > 0)
		{
			int _prevIndex = _currentIndex - 1;
			T _previous = value[_prevIndex];
			T _current = value[_currentIndex];
			
			value[_prevIndex] = _current;
			value[_currentIndex] = _previous;
		}
	}
	
	public static void SwapWithNext<T>(this List<T> value, int _currentIndex)
	{
		if (_currentIndex < value.Count - 1)
		{
			int _nextIndex = _currentIndex + 1;
			T _next = value[_nextIndex];
			T _current = value[_currentIndex];
			
			value[_nextIndex] = _current;
			value[_currentIndex] = _next;
		}
	}
	
	public static void SetActive(this tk2dAnimatedSprite value, bool _isActive)
	{
		if ( value == null )
			return;
		//if ( value.name == "Hit Effect" && value.transform.parent != null ) Debug.Log ( value.transform.parent.name + ", " + _isActive );
		if (value.Animator.playAutomatically) value.Animator.playAutomatically = false;
		
		if (_isActive) if (!value.renderer.enabled) value.renderer.enabled = true;
		
		value.gameObject.SetActive ( _isActive );
	}
	
	public static void SetActive(this tk2dSprite value, bool _isActive)
	{
		if ( value == null )
			return;
		
		if (_isActive) if (!value.renderer.enabled) value.renderer.enabled = true;
		
		value.gameObject.SetActive ( _isActive );
	}
	
	public static bool IsActive(this tk2dAnimatedSprite value)
	{
		return value.gameObject.activeSelf;
	}
	
	public static bool IsActive(this tk2dBaseSprite value)
	{
		return value.gameObject.activeSelf;
	}
	
	public static float Abs(this float value )
	{
		if (value < 0)
		{
			return (value * -1);
		}
		
		return value;
	}
	
	public static int Abs(this int value )
	{
		if (value < 0)
		{
			return (value * -1);
		}
		
		return value;
	}
	
	public static bool HasSprite ( this UIAtlas value, string _spriteName )
	{
		if ( value.spriteList != null && value.spriteList.Count > 0 )
		{
			for ( int i=0; i<value.spriteList.Count; i++ )
			{
				string _name = value.spriteList[i].name;
				if ( _name.Length == _spriteName.Length )
				{
					if ( _name.Equals ( _spriteName ) )
					{
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	/// <summary>
	/// Get array of colliders transform.
	/// </summary>
	/// <returns>
	/// The transforms.
	/// </returns>
	/// <param name='value'>
	/// Value.
	/// </param>
	public static Transform [] ToTransforms ( this Collider [] value )
	{
		Transform [] transforms = new Transform [ value.Length ];
		for ( int i = 0; i < value.Length; i++ )
		{
			transforms [ i ] = value [ i ].transform;
		}
		
		return transforms;
	}
	
	public static void Darken ( this UISprite value )
	{
		value.color = new Color ( 0.5f, 0.5f, 0.5f, 1.0f );
	}
	
	public static void Whiten ( this UISprite value )
	{
		value.color = Color.white;
	}
	
	public static Vector3[]	CirclePositions ( this Vector3 value, int _posAmt, float _radius )
	{
		// positioning the enemies
		Vector3[] circleDisplay = new Vector3[ _posAmt + 1 ];
		float angle = 360f / ( float ) _posAmt;
		for ( int i=0; i<_posAmt; i++ )
		{
			circleDisplay[i] = Quaternion.Euler ( 0, angle * i, 0 ) * new Vector3 ( -_radius, 0, _radius );
			circleDisplay[i] += value;
		}
		circleDisplay[ _posAmt ] = circleDisplay[0];
		
		return circleDisplay;
	}
	
}